<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Model\QuizModel;
use Carbon\Carbon;
use Illuminate\Support\Facades\Auth;

class QuizController extends Controller
{

    private $quizModel;

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        date_default_timezone_set('America/Los_Angeles');
        $this->quizModel = new QuizModel();
    }
    public function getQuizDetails()
    {
        $customerID = isset($_REQUEST['id']) ? $_REQUEST['id'] : '';
        $params = [];
        $shop    = Auth::user();

        $customerDetails = $shop->api()->rest('GET', '/admin/customers/' . $customerID . '.json', $params);
        $customerDetails = (object)$customerDetails;

        $customerDetails = $customerDetails->body;
        $email = isset($customerDetails->customer->email) ? $customerDetails->customer->email : '';
        $firstName = isset($customerDetails->customer->first_name) ? $customerDetails->customer->first_name : '';
        $lastName = isset($customerDetails->customer->last_name) ? $customerDetails->customer->last_name : '';
        $customerName = $firstName . " " . $lastName;

        $sessionIDs = $this->quizModel->getSessionID($email);

        $sessionIDs = $this->formatSessionID($sessionIDs);

        if (!empty($sessionIDs)) {
            $quizDetails = $this->quizModel->getQuizDetails($sessionIDs);
        }
        // echo"<pre>"; print_r($quizDetails);die('test');
        return view(
            'customerQuiz',
            [
                "customerName" => $customerName,
                "email" => $email,
                "quizCount" => isset($quizDetails) ? count($quizDetails) : 0,
                "quizDetails" => isset($quizDetails) ? $quizDetails : '',
            ]
        );
    }

    public function formatData($quizDetails)
    {
        $quizArray = [];
        if (!empty($quizDetails)) {
            foreach ($quizDetails as $quziData) {
                if (!isset($quizArray[$quziData->question])) {
                    $quizArray[$quziData->question] = $quziData->response;
                } else {
                    $data = $quizArray[$quziData->question] . " | " . $quziData->response;
                    $quizArray[$quziData->question] = $data;
                }
            }
        }
        return $quizArray;
    }

    public function formatSessionID($sessionIDs)
    {

        $sessionID = '';
        if (!empty($sessionIDs)) {
            foreach ($sessionIDs as $value) {
                $sessionID .= $value->session_id . ",";
            }
        }

        return rtrim($sessionID, ",");
    }

    public function getQuizDetailById()
    {

        $sessionID = isset($_REQUEST['sessionId']) ? $_REQUEST['sessionId'] : '';
        $email = isset($_REQUEST['email']) ? trim($_REQUEST['email']) : '';

        // $email = str_replace(" ", "+", $email);

        if (empty($sessionID)) {
            // $email = 'testing+TestUser@p80w.com';
            $sessionID = $this->quizModel->getSessionIds($email);  // TODO
        }

        $quizDetails = $this->quizModel->getQuizDetails($sessionID);
        //    echo"<pre>";print_r($quizDetails);die;
        $recommendations = $this->quizModel->getRecommendation($sessionID);

        $updatedDateTime = $quizDetails[$sessionID][0]->created_at;

        $updatedTime = new Carbon($updatedDateTime, 'America/Los_Angeles');
        $updatedTime = $updatedTime->format('c');

        // $updatedTime = Carbon::createFromFormat('Y-m-d H:i:s', $updatedDateTime, 'America/Los_Angeles');
        // $updatedTime = $updatedTime->setTimezone('UTC');

        $customFields = $this->quizModel->getCustomFields($sessionID);

        return view(
            'quizDetails',
            [
                "quizDetails" => $quizDetails[$sessionID],
                "sessionId" => $sessionID,
                "recommendation" => !empty($recommendations) ? json_decode($recommendations) : '',
                "quizCompletionTime" => $this->getQuizCompletionTime($quizDetails[$sessionID]),
                "quizTime" => $updatedTime,
                "email" => $email,
                "responseTime" => isset($customFields['responseTime']) ? $customFields['responseTime'] : '0.00'
            ]
        );
    }

    public function getCustomerDetails()
    {
        $email = isset($_REQUEST['email']) ? $_REQUEST['email'] : '';
        $customerName = isset($_REQUEST['name']) ? $_REQUEST['name'] : '';
        $sessionID = $this->quizModel->getSessionIds($email);  // TODO

        if (!empty($sessionID)) {
            $quizDetails = $this->quizModel->getQuizDetails($sessionID);
        }
        return view(
            'customerQuiz',
            [
                "customerName" => $customerName,
                "email" => $email,
                "quizCount" => isset($quizDetails) ? count($quizDetails) : 0,
                "quizDetails" => isset($quizDetails) ? $quizDetails : '',
            ]
        );
    }

    public function quizList(Request $request)
    {
        
        $limit = 20;
        $previousLink = '';
        $nextLink = '';
        $getPageLink = '';
        $shop    = Auth::user();
        
        $domain = $request->input('shop');
        $hmac =  $request->input('hmac');
        
        // dd($request->segment());
        
        // dd($_GET['page']);
        
        if (!isset($_GET['page'])) {
            $data = $shop->api()->rest(
                'GET', 
                '/admin/customers.json', 
                ["fields" => "email,first_name, last_name", "limit" => $limit]
            );
            $data = (object)$data;
            $previousLink = !empty($data->link->previous) ? $previousLink = $data->link->previous : '';
            $nextLink = !empty($data->link->next) ? $nextLink = $data->link->next : '';
        } else {
            $getPageLink = trim($_GET['page']);
            $data = $shop->api()->rest(
                'GET', 
                '/admin/customers.json', 
                ["limit" => $limit, 'page_info' => $getPageLink ]
            );
            $data = (object)$data;

            $previousLink = !empty($data->link->previous) ? $previousLink = $data->link->previous : '';
            $nextLink = !empty($data->link->next) ? $nextLink = $data->link->next : '';
        }

        $customerData = [];
        $customerEmail = [];
        foreach ($data->body->customers as $customer) {
            $email = $customer->email;
            $name = $customer->first_name . " " . $customer->last_name;
            $temp = array("email" => $email, "name" => $name);
            $customerData[] = $temp;
            $customerEmail[] = $email;
        }

        $emails = '"' . implode('","', $customerEmail) . '"';

        $quizDetails = $this->quizModel->getQuizDetailsByEmail($emails);

        $customerNames = $this->quizModel->getCustomerNamesByEmail($emails);

        $percentDetails = $this->getPercentDetails($quizDetails);

        return view('welcome', [
            "customers" => $customerData,
            "quizDetails" => $quizDetails,
            "previousLink" => $previousLink,
            "nextLink" => $nextLink,
            "percentDetails" => $percentDetails,
            "customerNames" => $customerNames,
            "domain" => $domain,
            "hmac" => $hmac
        ]);
    }

    public function getQuizCompletionTime($quizDetails)
    {
        if (!empty($quizDetails)) {
            $startTime = new \DateTime($quizDetails[0]->completion_time);
            $endTime = new \DateTime(end($quizDetails)->completion_time);

            $completionTime = $startTime->diff($endTime);
            return array("min" => $completionTime->i, "sec" => $completionTime->s);
        }
        return null;
    }

    public function getPercentDetails($quizDetails)
    {
        $percentDetails = array();
        if (!empty($quizDetails)) {
            foreach ($quizDetails as $email => $details) {
                $latestSessionID = end($details)['session_id'];
                $quizData = $this->quizModel->getQuizDetails($latestSessionID);
                $quizData = array_values($quizData);
                if ($quizData[0][0]->isrecommendation == 'Yes') {
                    $percentDetails[$email] = '100';
                } else {
                    $count = count($quizData[0]);
                    $percentDetails[$email] = ($count / 50) * 100;
                }
            }
        }
        return $percentDetails;
    }
    
    public function getPageList(Request $requestData) { 
        
        $limit = 20;
        $previousLink = '';
        $nextLink = '';
        $getPageLink = '';
        $pageLink = '';
        
        $shop    = Auth::user();
        // dd($requestData->page);
        if (empty($requestData)) {

            $response = array(
                "status"       => "Failed",
                "message" => "parameters are missing",
                "error" => "Bad request"
            );
            echo json_encode($response);
            die();
        }
        
        // dd($requestData);
        
        
        
        
        if(isset($requestData->page) || !empty($requestData->page)) {
            $pageLink  = $requestData->page;
        }
        // echo $pageLink;die('--------------');
        
        $data = $shop->api()->rest(
            'GET', 
            '/admin/api/2021-01/customers.json', 
            [ "limit" => $limit,"fields" => "email,first_name, last_name",'page_info' => $pageLink]
        );
        $data = (object)$data;

        $previousLink = !empty($data->link->previous) ? $previousLink = $data->link->previous : '';
        $nextLink = !empty($data->link->next) ? $nextLink = $data->link->next : '';

        $customerData = [];
        $customerEmail = [];
        
        foreach ($data->body->customers as $customer) {
            $email = $customer->email;
            $name = $customer->first_name . " " . $customer->last_name;
            $temp = array("email" => $email, "name" => $name);
            $customerData[] = $temp;
            $customerEmail[] = $email;
        }

        $emails = '"' . implode('","', $customerEmail) . '"';

        $quizDetails = $this->quizModel->getQuizDetailsByEmail($emails);

        $customerNames = $this->quizModel->getCustomerNamesByEmail($emails);

        $percentDetails = $this->getPercentDetails($quizDetails);

        

        $pageContent = '';

        if (isset($customerData) && !empty($customerData)) {
            foreach ($customerData as $customer) {
                // echo"<pre>";print_r($customers);die('test');
                $query_string = 'email=' . urlencode($customer['email']) . '&name=' . urlencode($customer['name']);
                $href = "customerData?" . $query_string;  //htmlentities           
                $pageContent .= '<tr class="Polaris-DataTable__TableRow">';
                $pageContent .= '<td class="Polaris-DataTable__Cell Polaris-DataTable__Cell--verticalAlignTop Polaris-DataTable__Cell--firstColumn" scope="row">';
                if (empty(trim($customer['name']))) {
                $pageContent .= $customer['name'];
                } else {
                $pageContent .= isset($customerNames[$customer['email']]) ? $customerNames[$customer['email']] : '';
                }
                $pageContent .='</td>';
                $pageContent .= '<td class="Polaris-DataTable__Cell Polaris-DataTable__Cell--verticalAlignTop Polaris-DataTable__Cell--numeric">'.$customer['email'].'</td>';
                $pageContent .= '<td class="Polaris-DataTable__Cell Polaris-DataTable__Cell--verticalAlignTop Polaris-DataTable__Cell--numeric">';
                $pageContent .= isset($quizDetails[$customer['email']]) ? count($quizDetails[$customer['email']]) : 0;
                $pageContent .= '</td>';
                $pageContent .= '<td class="Polaris-DataTable__Cell Polaris-DataTable__Cell--verticalAlignTop Polaris-DataTable__Cell--numeric quizCustomDate">';
                
                if(isset($quizDetails[$customer['email']]) && count($quizDetails[$customer['email']]) > 1 ) {
                    
                    $createdAt = new Carbon(end($quizDetails[$customer['email']])['created_at'], 'America/Los_Angeles');
                    $createdAt = $createdAt->toDayDateTimeString();
                    $pageContent .= $createdAt ;
                        
                } else if (isset($quizDetails[$customer['email']]) && count($quizDetails[$customer['email']]) == 1){
                    
                    $createdAt = new Carbon(reset($quizDetails[$customer['email']])['created_at'], 'America/Los_Angeles');
                    $createdAt = $createdAt->toDayDateTimeString();
                    $pageContent .= $createdAt ;
                    
                } else {
                    
                    $pageContent .= "";
                }
                
                $pageContent .= '</td>';
                $pageContent .= '<td class="Polaris-DataTable__Cell Polaris-DataTable__Cell--verticalAlignTop Polaris-DataTable__Cell--numeric">';
                    
                if(isset($percentDetails[$customer['email']])) { 
                    
                    if($percentDetails[$customer['email']] == '100') { 
                        //$pageContent .= $percentDetails[$customer['email']];
                        $pageContent .= '<div class="progress-circle over50 green p100">
                                    <span> 100% </span>
                                    <div class="left-half-clipper">
                                    <div class="first50-bar"></div>
                                    <div class="value-bar"></div>
                                    </div>
                                </div> </td>';

                    } else {  

                        if( $percentDetails[$customer['email']] <= 49) { 
                            //$pageContent .= $percentDetails[$customer['email']];
                            $pageContent .= '<div class="progress-circle red p'.$percentDetails[$customer['email']].'">
                                <span>'. $percentDetails[$customer['email']].'% </span>
                                <div class="left-half-clipper">
                                <div class="first50-bar"></div>
                                <div class="value-bar"></div>
                                </div>
                            </div>'; 
                        }
                                                                            
                        if( $percentDetails[$customer['email']] >= 50 && $percentDetails[$customer['email']] <= 89) { 

                            //$pageContent .= $percentDetails[$customer['email']];
                            $pageContent .= '<div class="progress-circle over50 orange p'.$percentDetails[$customer['email']].'">
                            <span>'.$percentDetails[$customer['email']].'% </span>
                            <div class="left-half-clipper">
                                <div class="first50-bar"></div>
                                <div class="value-bar"></div>
                            </div>
                            </div>'; 

                        } 

                        if( $percentDetails[$customer['email']]  >= 90 ) {

                            //$pageContent .= $percentDetails[$customer['email']];
                            $pageContent .= '<div class="progress-circle over50 green p'.$percentDetails[$customer['email']].'">
                                <span>'.$percentDetails[$customer['email']].'% </span>
                                <div class="left-half-clipper">
                                <div class="first50-bar"></div>
                                <div class="value-bar"></div>
                                </div>
                            </div>'; 

                        }
                    }


                    $pageContent .= '</td>';
                } else { 
                    
                    $pageContent .= '<div class="progress-circle red p0">
                        <span> 0% </span>
                        <div class="left-half-clipper">
                        <div class="first50-bar"></div>
                        <div class="value-bar"></div>
                        </div>
                    </div> </td>';
                } 
                    
                $pageContent .= '<td class="Polaris-DataTable__Cell Polaris-DataTable__Cell--verticalAlignTop Polaris-DataTable__Cell--numeric">';
                $pageContent .= '<div style="--top-bar-background:#00848e; --top-bar-background-lighter:#1d9ba4; --top-bar-color:#f9fafb; --p-frame-offset:0px;">';
                $pageContent .= '<a class="Polaris-Link btn-quizView" href="'.$href.'" data-polaris-unstyled="true" data-sessionId="" data-email="'.$customer['email'].'" >';
                $pageContent .= 'View Details';
                $pageContent .= '</a>';
                $pageContent .= '</div> </td></tr>';
            }
        }
        
        // $view = view('welcome', [
        //     "customers" => $customerData,
        //     "quizDetails" => $quizDetails,
        //     "previousLink" => $previousLink,
        //     "nextLink" => $nextLink,
        //     "percentDetails" => $percentDetails,
        //     "customerNames" => $customerNames
        // ])->render();
        
        // return response()->json(['html'=>$view]);

        $returnData =  [
            "previousLink" => $previousLink,
            "nextLink" => $nextLink,            
            "pageContent" => $pageContent,
        ];
        
        return json_encode($returnData);

        // return view('welcome', [
        //     "customers" => $customerData,
        //     "quizDetails" => $quizDetails,
        //     "previousLink" => $previousLink,
        //     "nextLink" => $nextLink,
        //     "percentDetails" => $percentDetails,
        //     "customerNames" => $customerNames
        // ]);

    }
}
